@@ -4,8 +4,7 @@ import sklearn.model_selection
 from matplotlib import pyplot as plt
 from sklearn.tree import DecisionTreeRegressor
 
-from pytolemaic.pytrust import PyTrust
-from pytolemaic.utils.dmd import DMD
+from pytolemaic import PyTrust
 
 
 def run():
@@ -39,22 +38,18 @@ def run():
         model=estimator,
         xtrain=xtrain, ytrain=ytrain,
         xtest=xtest, ytest=ytest,
-        columns_meta={DMD.FEATURE_NAMES: feature_names},
+        feature_names=feature_names,
         metric='mae')
-    scoring_report = pytrust.create_scoring_report()
-    scoring_report.plot()
+    pytrust.scoring_report.plot()
 
-    sensitivity_report = pytrust.create_sensitivity_report()
-    sensitivity_report.plot()
+    pytrust.sensitivity_report.plot()
 
-    dataset_analysis_report = pytrust.create_dataset_analysis_report()
-    dataset_analysis_report.plot()
+    pytrust.dataset_analysis_report.plot()
 
-    quality_report = pytrust.create_quality_report()
-    quality_report.plot()
+    pytrust.quality_report.plot()
 
     sample = xtest[0, :].reshape(1, -1)
-    explainer = pytrust.create_lime_explainer(max_samples=64000)
+    explainer = pytrust.create_lime_explainer(max_samples=16000)
     explainer.explain(sample=sample)
 
     uncertainty_model = pytrust.create_uncertainty_model(method='default')
